from setuphelpers import *

"""
Installation procedures:
https://mozilla.github.io/policy-templates/
https://mozilla.github.io/policy-templates/#extensionsettings

"""
policies_file = "policies.json"


def get_firefox_policies_dirs():
    detected_apps = installed_softwares("Firefox")
    firefox_policies_dirs = []
    if get_os_name() == "Linux":
        firefox_policies_dirs.append("/etc/firefox/policies")
    else:
        if not detected_apps:
            error("Firefox is not installed on this machine.")
        for ffox in detected_apps:
            if get_os_name() == "Windows":
                firefox_policies_dirs.append(makepath(ffox["install_location"], "distribution"))
            if get_os_name() == "Darwin":
                firefox_policies_dirs.append(makepath(ffox["install_location"], "Contents", "Resources", "distribution"))

    return firefox_policies_dirs


def install():
    # Check if policies.json is valid
    try:
        json_load_file(policies_file)
    except:
        error(f"{policies_file} is malformed.")

    # Interact differently with Luti
    if params.get("running_as_luti", False):
        WAPT.install("tis-firefox")

    for firefox_policies_dir in get_firefox_policies_dirs():
        policies_path = makepath(firefox_policies_dir, policies_file)
        print(f"Writting Firefox configuration to: {policies_path}")
        killalltasks(ensure_list(control.impacted_process))
        if get_os_name() == "Darwin":
            run("sudo defaults write /Library/Preferences/org.mozilla.firefox EnterprisePoliciesEnabled -bool TRUE")
        mkdirs(firefox_policies_dir)
        filecopyto(policies_file, policies_path)

    if get_os_name() == "Windows":
        ffox_policies_reg_path = r"SOFTWARE\Policies\Mozilla\Firefox"
        if reg_key_exists(HKEY_LOCAL_MACHINE, ffox_policies_reg_path):
            print(
                "WARNING: Registry policies has been removed because it bypasses the policies.json file (source: https://support.mozilla.org/bm/questions/1236197)."
            )
            registry_deletekey(HKEY_LOCAL_MACHINE, ffox_policies_reg_path, "", force=True)

    # Forget package instead of adding it as conflict
    WAPT.forget_packages(f"{control.package.split('-')[0]}-config-firefox")


def uninstall():
    for firefox_policies_dir in get_firefox_policies_dirs():
        policies_path = makepath(firefox_policies_dir, policies_file)
        remove_file(policies_path)

    if get_os_name() == "Windows":
        # Reapplying minimal configuration for Firefox
        if not isfile(policies_path):
            registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Mozilla\Firefox", "DisableAppUpdate", 1, REG_DWORD)
            registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Mozilla\Firefox", "DisableTelemetry", 1, REG_DWORD)


def audit():
    audit_status = "OK"
    for firefox_policies_dir in get_firefox_policies_dirs():
        policies_path = makepath(firefox_policies_dir, policies_file)
        if not isfile(policies_path):
            print("WARNING: Configuration is NOT applied for Firefox, re-applying...")
            install()
            audit_status = "WARNING"
        else:
            print("OK: Configuration is correctly applied for Firefox.")
            audit_status = "OK"

    return audit_status


# def merge_dicts(dict1, dict2):
#     for key, value in dict2.items():
#         if key in dict1 and isinstance(value, dict) and isinstance(dict1[key], dict):
#             merge_dicts(dict1[key], value)  # recursive merge
#         else:
#             dict1[key] = value
#     return dict1
